xquery version "3.1";

(:~
 : This module controls the serialization of any documents of the KMG.
 : Documents are given over by the TextGrid connector, prepared here, and given
 : over to the necessary functions for transformation/storage.
 : 
 : @author Simon Sendler
 : @version 1.0
 : @since 1.0
 :)
module namespace kmg-serial="http://mollenhauer-edition.de/ns/kmg-serial";

import module namespace kmg-register-transformation="https://mollenhauer-edition.de/ns/register-transformation" at "register-transformation.xqm";
import module namespace bdn-utils="http://bdn-edition.de/ns/utils" at "../bdn/utils.xqm";
import module namespace ifweb="http://bdn-edition.de/intermediate_format/ifweb" at "/db/apps/interformat/modules/intermediate_format/ifweb.xql";
import module namespace xml-utils="http://bdn-edition.de/portal/xml-utils" at "/db/apps/sade/modules/bdn/xml-utils.xqm ";
import module namespace bdn-odd="http://bdn-edition.de/ns/odd" at "../bdn/schema.xqm";
import module namespace config="https://sade.textgrid.de/ns/config" at "/db/apps/sade/config.xqm";

declare namespace tei="http://www.tei-c.org/ns/1.0";

(:~
 : This function assures that any directories necessary to the serialization are
 : available. 
 :)
declare function kmg-serial:directoryHelper() {
    let $resources := bdn-utils:assure-dir-available("resources")
    let $edition := bdn-utils:assure-dir-available("edition")
    let $edition := bdn-utils:assure-dir-available("lido")
    return
        true()
};

(:~
 : This function injects the Series Statement, which is stored in a seperate
 : document in TextGrid and referenced in the edition, into edition documents. 
 : This change is stored, and not done in memory during transformation, as
 : complete XML-documents should be offered for download.
 : 
 : @param $doc A TEI-XML document
 : @param $log A logfile
 : @return $doc A TEI-XML document with the full Series Statement
 :)
declare function kmg-serial:inject-seriesStmt($doc, $log) {
    try {
            update replace $doc//tei:teiHeader//tei:seriesStmt
            with fn:doc($config:sade-data || "3qq81.xml")//*,
            bdn-utils:add-log-entry($log, "Injection of seriesStmt: Success."),
            $doc
        }
        catch * {
            bdn-utils:add-log-entry($log, "Injection of seriesStmt: Failed. Some information might not be displayed correctly."),
                $doc
        }
};

(:~
 : This function coordinates the serialization of edition documents. And hands
 : the document over to any necessary functions.
 : 
 : @param $doc A TEI-XML document
 :)
declare function kmg-serial:serialize-edition($doc) {
    let $filename := util:document-name($doc)
    let $log := bdn-utils:create-log("Mollenhauer-" || $filename)
    let $doc := kmg-serial:inject-seriesStmt($doc, $log)

    let $single := xml-utils:make-html-single-view($doc, $filename, $log)
    let $digital := xml-utils:make-html-digital-view($doc, $filename, $log)
    (: TODO: implement handling of textual commentary :)
    let $commentary := () 
    return
        (true())
};

(:~
 : This function handles the storage of LIDO-XML.
 : 
 : @param $doc A LIDO-XML document
 : @param $uri The TextGrid-URI of the document
 :)
declare function kmg-serial:serialize-lido($doc, $uri) {
    let $content := $doc
    let $result := xmldb:store($config:lido, $uri, $content)
        return
            true()
};

(:~
 : This function receives documents from the TextGrid connector, identifies them
 : and hands them over to their respective functions/modules for further
 : processing.
 : 
 : @param $uri A TextGrid-URI of an XML document
 :)
declare function kmg-serial:serialize($uri) {
    let $prepare := kmg-serial:directoryHelper()
    let $doc := doc($config:sade-data || $uri)

    (: run/update register.xml :)
    let $register_dir := xmldb:create-collection("/db/data", "resources")
    let $write_register := xmldb:store("/db/data/resources/", "persons.xml", kmg-register-transformation:return-register($config:sade-data))
    
    
    return
        if ( $uri = "3r8zv.xml" ) then
            let $odd := bdn-odd:main()
            return
                true()
        else if ( $uri = "3qq81.xml" ) then
            true()
        else if ( name($doc/*) = "lido:lido" ) then
            kmg-serial:serialize-lido($doc, $uri)
        else
            kmg-serial:serialize-edition($doc)
};