module namespace kmg-register-transformation="https://mollenhauer-edition.de/ns/register-transformation";

declare namespace tei="http://www.tei-c.org/ns/1.0";
declare namespace pnd="http://textgrid.info/namespaces/vocabularies/pnd";


declare function kmg-register-transformation:all-references($sade-data-path as xs:string*){

    for $doc in collection($sade-data-path)
    let $persons := $doc//tei:text//tei:name[@type='person']
    return 
        for $person in $persons
        return
            (: if $person/@ref is a reference to xml:id replace the hashtag 
            and search for xml:id :)
            if (matches($person/@ref, '^#')) then
                    $doc//.[@xml:id=replace($person/@ref, '#', '')]
            else
                $person
};



declare function kmg-register-transformation:unique-references($sade-data-path as xs:string*){
    
    for $person_ref in distinct-values(kmg-register-transformation:all-references($sade-data-path)/@ref) 
    let $stripped_id := tokenize($person_ref, ":")[2]
    return
        try{
            <person ref="{$person_ref}">{doc("https://ref.de.dariah.eu/pndsearch/pndquery.xql?id=" || $stripped_id)//pnd:si[1]/text()}</person>
        } catch * {
            <person ref="{$person_ref}">_Wrong GND-id format!</person>
        }
};


declare function kmg-register-transformation:return-register($sade-data-path as xs:string*){
    let $register :=
        <TEI>
            <teiHeader>
                <fileDesc>
                    <titleStmt>
                        <title>Register for persons</title>
                        <author>
                            <name ref="gnd:118583352" type="person">Klaus Mollenhauer</name>
                        </author>
                        <funder>Deutsche Forschungsgemeinschaft (DFG)</funder>
                        <respStmt>
                            <persName>
                                <forename/>
                                <surname/>
                            </persName>
                            <resp/>
                        </respStmt>
                    </titleStmt>
                    <editionStmt>
                        <edition>
                            <edition>Historisch-kritische Edition</edition>
                        </edition>
                    </editionStmt>
                    <extent>File size</extent>
                    <publicationStmt>
                        <publisher>
                            <name type="org">Niedersächsische Staats- und
                        Universitätsbibliothek Göttingen</name>
                        </publisher>
                        <pubPlace>
                            <name ref="geo:2918632" type="place" xml:id="GÖ">Göttingen</name>
                        </pubPlace>
                        <date type="publication"/>
                        <availability>
                            <licence target="https://creativecommons.org/licenses/by-sa/4.0/">Distributed under the Creative Commons                                 Attribution-ShareAlike 4.0 Generic (International) License.</licence>
                        </availability>
                    </publicationStmt>
                    <sourceDesc> </sourceDesc>
                </fileDesc>
                <encodingDesc>
                    <projectDesc>
                        <p>Klaus Mollenhauer Gesamtausgabe</p>
                    </projectDesc>
                    <editorialDecl>
                        <correction>
                            <p>whether corrections are encoded or not</p>
                        </correction>
                        <hyphenation>
                            <p>whether we respect the caesura at the end of lines</p>
                        </hyphenation>
                        <normalization>
                            <p>Information about abbreviations</p>
                        </normalization>
                    </editorialDecl>
                </encodingDesc>
                <profileDesc>
                    <langUsage>
                        <language ident="deu">Deutsch</language>
                    </langUsage>
                </profileDesc>
            </teiHeader> 
            <text>
                <body>
                    <listPerson>
                    {kmg-register-transformation:unique-references($sade-data-path)}
                    </listPerson>
                </body>
            </text>
        </TEI>
        
        return
            $register
};



